/* Zeta API - Z/hardware/machine/console/Game Boy.h
  ______ _____   _______  ______   _____   _________  ___
 /  ___//  _  | /       \/  ___/  /  _  ] /  __  \  \/  /
/  /_  /  /-' |/  /  /  /  __/   /  __  \/  /_/  /\    /
\_____/__/ |__|__/__/__/_____/  /_______/\______/ /___/ TM
Copyright (C) 2006-2022 Manuel Sainz de Baranda y Goñi.
Released under the terms of the GNU Lesser General Public License v3.


 ________________________________________
|					 |
|      Name: Game Boy			 |
|    Vendor: Nintendo			 |
|  Released: 1989-04-21 (JP)		 |
|	     1989-07-31 (North America)	 |
|	     1990-09-28 (Europe)	 |
|	CPU: Sharp LR35902 @ 4.19 MHz	 |
|     Video:				 |
|     Audio:				 |
|					 |
'========================================'

Front view		Back view
 _n__________________	 __________________n_
|_|________________|_|	|   |		=|   |
|  ________________  |	|   |		 |   |
| |   __________   | |	| O |		 | O |
| | .|		|  | |	|   |____________|   |
| |  |		|  | |	|  |		  |  |
| |  |		|  | |	|  |   G0502050   |  |
| |  |		|  | |	|  |  |	       |  |  |
| |  '__________'  | |	|  |  |________|  |  |
| '_______________/  |	| O|		  |O |
|   _ GAME BOY™	     |	|==|=====.--.=====|==|
| _| |_		  _  |	|==|.----|  |----.|==|
||_ O _|       _ (_) |	|==||============||==|
|  |_|	      (_)  A |	|==||============||==|
|    _	_	B    |	|==||============||==|
|   // //	     |	|==||============||==|
|	     \\\\\\  |	|  ||		 ||  |
|	      \\\\\\ /	\  '|____________|'  |
'_________..________/	 \___________________'  */


#ifndef Z_hardware_machine_console_Game_Boy_H
#define Z_hardware_machine_console_Game_Boy_H

/* MARK: - Memory map */

/*----------------------------------------------------------------------.
| Region    | Size | Type | R/W | Content				|
|-----------+------+------+-----+---------------------------------------|
| 0000-3FFF | 4000 | ROM  | R   | Cartridge ROM bank #0			|
| 4000-7FFF | 4000 | ROM  | R   | Cartridge ROM selected bank		|
|-----------+------+------+-----+---------------------------------------|
| 8000-9FFF | 2000 | VRAM |     | Video RAM				|
| A000-BFFF | 2000 | RAM  |     | Cartridge RAM				|
| C000-CFFF | 1000 | RAM  |     | Work RAM (WRAM) bank #0		|
| D000-DFFF | 1000 | RAM  |     | Work RAM (WRAM) selected bank (#1-#7) |
| E000-FDFF | 1E00 | RAM  |     | Echo RAM (ERAM) (mirror of C000-DDFF) |
|-----------+------+------+-----+---------------------------------------|
| FE00-FE9F |   A0 |      |     | Sprite attribute table (OAM)		|
|-----------+------+------+-----+---------------------------------------|
| FEA0-FEFF |   60 | N/A  |     | Not used				|
|-----------+------+------+-----+---------------------------------------|
| FF00-FF7F |   80 | MMIO |     | Memory mapped I/O registers		|
|-----------+------+------+-----+---------------------------------------|
| FF80-FFFE |   7F | RAM  |     | High RAM (HRAM)			|
|-----------+------+------+-----+---------------------------------------|
| FFFF	    |    1 | MMIO |     | Interrupts Enable Register (IE)	|
'======================================================================*/

#define Z_GAME_BOY_ADDRESS_ROM	0
#define Z_GAME_BOY_ADDRESS_VRAM 0x8000
#define Z_GAME_BOY_ADDRESS_WRAM	0xC000
#define Z_GAME_BOY_ADDRESS_ERAM	0xE000
#define Z_GAME_BOY_ADDRESS_OAM	0xFE00
#define Z_GAME_BOY_ADDRESS_HRAM 0xFF80
#define Z_GAME_BOY_MMIO_IE	0xFFFF

/* MARK: - Screen */

#define Z_GAME_BOY_SCREEN_FPS	 59.7
#define Z_GAME_BOY_SCREEN_WIDTH	 160
#define Z_GAME_BOY_SCREEN_HEIGHT 144
#define Z_GAME_BOY_SCREEN_PIXELS 23040 /* 160 * 144 */

/* MARK: - Timings */

#define Z_GAME_BOY_CPU_HZ
#define Z_GAME_BOY_PPU_HZ
#define Z_GAME_BOY_APU_HZ

#endif /* Z_hardware_machine_console_Game_Boy_H */
