/* Zeta API - Z/hardware/machine/console/NES.h
 ___  __ __       __              __
|   \|  |__|____ |  | ____ ____ _|  |____
|  , `  |  |    |_  _|  -_|    | _  |  _ |
|__|\___|__|__|_||__||____|__|_|__,_|____|
 E N T E R T A I N M E N T    S Y S T E M
Copyright (C) 2006-2022 Manuel Sainz de Baranda y Goñi.
Released under the terms of the GNU Lesser General Public License v3.


 _________________________________________________________
|							  |
|      Name: Family Computer (aka Famicom)		  |
|     Model: HVC-001					  |
|    Vendor: Nintendo					  |
|  Released: 1983-07-15 (JP)				  |
|	CPU: Ricoh RP2A03 @ ~1.789773 MHz		  |
|	RAM: 2 KiB (+ cartridge RAM)			  |
|     Video: Ricoh RP2C02 @ ~5.369318 MHz (VDC, aka PPU)  |
|     Sound: Integrated in the CPU chip (PSG, aka APU)	  |
|							  |
'========================================================='


HVC-001: Top view
   ____________________________________
  |	 |  | | | | | | | | | | |      |
 _|	 |  | | | | | | | | | | |      |_
|  |_____|  |_|_|_|_|_|_|_|_|_| |_____|  |
|  |	 |			|     |  |
|  |	 |______________________|     |  |
|  | _______________o________________ |  |
|  ||--------------------------------||  |
|  ||				     ||  |
|  ||==============----==============||  |
|  ||________________________________||  |
|  |   _____	__________    _____   |  |
|  |= |._.. |  |    ||	  |  |._._ | =|  |
|  |  | .-_ |  |    ||	  |  |.-...|  |  |
|  |  '--|--'  |    ||	  |  '--|--'  |  |
'__'	 v     |    ||	  |	v     '__'
  | ,---------.|.--------.|,---------. |
  | |     .--.||`========'||.--.     | |
  | |POWER|__|||  EJECT   |||__|RESET| |
  | |_________||__________||_________| |
  |____________________________________|
  | __________________________________ |
  | \	      FAMILY COMPUTER	     / |
  |  `=============================='  |
  `------------------------------------'


 _________________________________________________________
|							  |
|    Name: Nintendo Entertainment System		  |
|   Model: NES-001					  |
|  Vendor: Nintendo					  |
|     RAM: 2 KiB (+ cartridge RAM)			  |
|   Sound: Integrated in the CPU chip (PSG, aka APU)	  |
|							  |
|  NES-001 (NTSC)					  |
|  ==============					  |
|  Released: 1985-10-18 (KR-US)				  |
|	     1986       (CA)				  |
|	     1987       (Australasia)			  |
|	CPU: Ricoh RP2A03 @ ~1.789773 MHz		  |
|     Video: Ricoh RP2C02 @ ~5.369318 MHz (VDC, aka PPU)  |
|							  |
|  NES-001 (PAL)					  |
|  =============					  |
|  Released: 1986-09-01 (DE-DK-FR-NL-NO-SE)		  |
|	     1987       (AU-IE-IT-NZ-UK)		  |
|	CPU: Ricoh RP2A07 @ 1.66260703125 MHz		  |
|     Video: Ricoh RP2C07 @ 5.3203425 MHz (VDC, aka PPU)  |
|							  |
'========================================================='


NES-001: Front view
 ___ __________________________________ ______ ______         .---------.
|   |				       |______|	     |        |.=====.  |
|   | Nintendo®			       |      |	     |   GND ->|| O  \\ |
|   | ENTERTAINMENT SYSTEM™	       |      |	     |   CLK ->|| O O ||<- +5V
|   |__________________________________|______|	     |   OUT ->|| O O ||<- D3
|______________________________________|______|______|    D0 ->|| O O ||<- D4
|   |	 _____	 _____	|	       | 1  2 |	     |        |'======' |
 \  | o [POWER] [RESET] |	       ||\ |\ |	    /         '---------'
  \ |___________________|	       ||_||_||	   /      Controller Connector
   \___________________________________|______|___/             (Female)


NES-001: Back view
 ______ ______ ______________________________________
|      |______|					     |
|      |      |					     |
|      |      |					     |
|      |______|					     |
|______|______|______________________________________|
|   | o   [] O  |				     |
 \  |___________|				    /
  \ | AC  CH RF	|		 __		   /
   \|___________|_______________/__\______________/


NES-001: Top view
 ______________________________________ ______ ______
|				       |      |	     |
|				       |______|	     |
|				       |      |	     |
|				       |======|	     |
|				       |======|	     |
|				       |======|	     |
|				       |======|	     |
|				       |======|	     |
|				       |======|	     |
|				       |======|	     |
|				       |======|	     |
|				       |======|	     |
|				       |======|	     |
|				       |======|	     |
|				       |======|	     |
|				       |======|	     |
|    __________________________________|======|______|
|   |				       |      |	     |
|   |				       |______|	     |
|   |				       |      |	     |
'___|__________________________________'______'______'


NES-001: Bottom view
 ____________________________________________________
| O ||_n____n_n__|	      O	|  |		 O   |
|   |  _     ________________	|  |  ______  _  |   |
|---| (_)   |		     |	|  | |	    |(_) |   |
|   |	    |		     |	|  | |______|	 |   |
|  [|=====  |		     |	|  |  _________	 |   |
|   |=====  |________________|	|  | |	       | |   |
|  [|				|  | |_________| |   |
|   |				|  |		 |   |
|---|				|  |		 |   |
|   |				|  |		 |   |
|   |=====		  ______|  |_____   =====|   |
|   |=====		 |.-------------.|  =====|   |
|   |=====		 ||   _______	||  =====|   |
|   --------------------- |  |_______|	|'--------   |
|   _____________________ |_____________| ________   |
|   |			 |_______________|	 |   |
|   |						 |   |
|   |  _				      _  |   |
|   | (_)				     (_) |   |
|   |	  ______	      O			 |   |
'___O____|_o__o_|________________________________O___'


NES-001: Controller
		 . .
 ________________|_|_________________________________
|						     |
|  ________________________________________________  |
| |		    |_____________|		   | |
| |	  ___	     _____________		   | |
| |	 |   |	    |_____________|   Nintendo®	   | |
| |   ___| | |___    _____________		   | |
| |  | __ ,-. __ |  |SELECT__START|   ___   ___	   | |
| |  |___ `-' ___|   _____________   |,-.| |,-.|   | |
| |	 | | |	    |_(___)_(___)_|  |._,| |._,|   | |
| |	 |___|	     _____________   '---B '---A   | |
| |_________________|_____________|________________| |
'____________________________________________________'


 ______________________________________________
|					       |
|      Name: Dendy			       |
|     Model:				       |
|    Vendor:				       |
|  Released:				       |
|	CPU: UMC UA6527P @ ~1.773447 MHz       |
|	RAM: 2 KiB (+ cartridge RAM)	       |
|     Video: UMC UA6538 @ 5.3203425 MHz (VDC)  |
|     Sound:				       |
|					       |
|  Dendy Junior				       |
|  ============				       |
|					       |
|  Dendy Junior II			       |
|  ===============			       |
|					       |
|					       |
'=============================================='


Dendy Junior
   ____________________________________
  |	 |  | | | | | | | | | | |      |
 _|	 |  | | | | | | | | | | |      |_
|  |_____|  |_|_|_|_|_|_|_|_|_| |_____|  |
|  |	 |			|     |  |
|  |	 |______________________|     |  |
|  | ________________________________ |  |
|  ||--------------------------------||  |
|  ||				     ||  |
|  ||==============----==============||  |
|  ||________________________________||  |
|  |   _____	__________    _____   |  |
|  |= |._.. |  |    ||	  |  |._._ | =|  |
|  |  | .-_ |  |    ||	  |  |.-...|  |  |
|  |  '--|--'  |    ||	  |  '--|--'  |  |
'__'	 v     |    ||	  |	v     '__'
  | ,---------.|.--------.|,---------. |
  | |     .--.||`========'||.--.     | |
  | |POWER|__|||  EJECT   |||__|RESET| |
  | |_________||__________||_________| |
  |____________________________________|
  | __________________________________ |
  | \--Dendy-----JUNIOR-------,(S),--/ |
  |  `=============================='  |
  `------------------------------------'


Dendy Junior II
   ____________________________________
  |	 |  | | | | | | | | | | |      |
 _|	 |  | | | | | | | | | | |      |_
|  |_____|  |_|_|_|_|_|_|_|_|_| |_____|  |
|  |	 |			|     |  |
|  |	 |______________________|     |  |
|  | _______________o________________ |  |
|  ||--------------------------------||  |
|  ||				     ||  |
|  ||==============----==============||  |
|  ||________________________________||  |
|  |   _____	__________    _____   |  |
|  |= |._.. |  |    ||	  |  |._._ | =|  |
|  |  | .-_ |  |    ||	  |  |.-...|  |  |
|  |  '--|--'  |    ||	  |  '--|--'  |  |
'__'	 v     |    ||	  |	v     '__'
  | ,---------.|.--------.|,---------. |
  | |     .--.||`========'||.--.     | |
  | |POWER|__|||  EJECT   |||__|RESET| |
  | |_________||__________||_________| |
  |____________________________________|
  | __________________________________ |
  | \--Dendy-----JUNIOR-------,(S),--/ |
  |  `=============================='  |
  `------------------------------------'
*/


#ifndef Z_hardware_machine_console_NES_H
#define Z_hardware_machine_console_NES_H

#include <Z/macros/language.h>
#include <Z/macros/member.h>
#include <Z/types/integral.h>

/* MARK: - Memory map */

/*------------------------------------------------------------------.
| Region    | Size | Type | R/W | Content			    |
|-----------+------+------+-----+-----------------------------------|
| 0000-07FF |  800 | RAM  | R/W | Internal RAM			    |
|-----------+------+------+-----+-----------------------------------|
| 0800-1FFF |  800 | RAM  | R/W | Mirror of 0000-07FF *3	    |
|-----------+------+------+-----+-----------------------------------|
| 2000	    |    1 | MMIO |   W | PPU: Control			    |
| 2001	    |    1 | MMIO |   W | PPU: Mask			    |
| 2002	    |    1 | MMIO | R   | PPU: Status			    |
| 2003	    |    1 | MMIO |   W | PPU: OAM address		    |
| 2004	    |    1 | MMIO | R/W | PPU: OAM data			    |
| 2005	    |    1 | MMIO |   W | PPU: Scroll			    |
| 2006	    |    1 | MMIO |   W | PPU: Address			    |
| 2007	    |    1 | MMIO | R/W | PPU: Data			    |
|-----------+------+------+-----+-----------------------------------|
| 2008-3FFF | 1FF8 | MMIO |  =  | Mirror of 2000-2007 *1023	    |
|-----------+------+------+-----+-----------------------------------|
| 4000	    |    1 | MMIO |  -	|				    |
| 4001	    |    1 | MMIO |  -	|				    |
| 4002	    |    1 | MMIO |  -	|				    |
| 4003	    |    1 | MMIO |  -	|				    |
| 4004	    |    1 | MMIO |  -	|				    |
| 4005	    |    1 | MMIO |  -	|				    |
| 4006	    |    1 | MMIO |  -	|				    |
| 4007	    |    1 | MMIO |  -	|				    |
| 4008	    |    1 | MMIO |  -	|				    |
| 4009	    |    1 |	  |  -	| Not used			    |
| 400A	    |    1 | MMIO |  -	|				    |
| 400B	    |    1 | MMIO |  -	|				    |
| 400C	    |    1 | MMIO |  -	|				    |
| 400D	    |    1 |	  |  -	| Not used / Unknown		    |
| 400E	    |    1 | MMIO |  -	|				    |
| 400F	    |    1 | MMIO |  -	|				    |
| 4010	    |    1 | MMIO |   W | APU: DMC control		    |
| 4011	    |    1 | MMIO |   W | APU: DMC direct load		    |
| 4012	    |    1 | MMIO |   W | APU: DMC sample address	    |
| 4013	    |    1 | MMIO |   W | APU: DMC sample size		    |
| 4014	    |    1 | MMIO |   W | PPU: OAM DMA			    |
| 4015	    |    1 | MMIO | R/W | APU: Status/Control		    |
| 4016	    |    1 | MMIO | R/W | Controller 1 /		    |
| 4017	    |    1 | MMIO | R/W | Controller 2 / APU: Frame counter |
| 4018	    |	 1 | MMIO | R	|				    |
| 4019	    |	 1 | MMIO | R	|				    |
| 401A	    |	 1 | MMIO | R/W |				    |
| 401B	    |	 1 | MMIO | ?	| ?				    |
| 401C	    |	 1 | MMIO | R/W |				    |
| 401D	    |	 1 | MMIO | R/W |				    |
| 401E	    |	 1 | MMIO | R/W |				    |
| 401F	    |	 1 | MMIO | ?   | ...				    |
|-----------+------+------+-----+-----------------------------------|
| 4020-5FFF | BFE0 |      |     | Cartridge: Expansion ROM/RAM/MMIO |
| 6000-7FFF | 2000 | RAM  | R/W | Cartridge: WRAM		    |
| 8000-FFFF | 8000 | ROM  | R	| Cartridge: PRG-ROM 		    |
'------------------------------------------------------------------*/

#define Z_NES_ADDRESS_RAM     0
#define Z_NES_ADDRESS_MMIO    0x2000
#define Z_NES_ADDRESS_WRAM    0x6000
#define Z_NES_ADDRESS_PRG_ROM 0x8000

#define Z_NES_SIZE_RAM	            0x800  /*  2 KiB */
#define Z_NES_SIZE_WRAM	            0x2000 /*  8 KiB */
#define Z_NES_SIZE_PRG_ROM_APERTURE 0x8000 /* 32 KiB */
#define Z_NES_SIZE_PRG_ROM_BANK	    8192
#define Z_NES_SIZE_CHR_ROM_BANK	    1024

/* MARK: Memory-maped I/O */

#define Z_NES_MMIO_PPU_CONTROL		  0x2000 /* write      */
#define Z_NES_MMIO_PPU_MASK		  0x2001 /* write      */
#define Z_NES_MMIO_PPU_STATUS		  0x2002 /* read       */
#define Z_NES_MMIO_PPU_OAM_ADDRESS	  0x2003 /* write      */
#define Z_NES_MMIO_PPU_OAM_DATA		  0x2004 /* read/write */
#define Z_NES_MMIO_PPU_SCROLL		  0x2005 /* write      */
#define Z_NES_MMIO_PPU_ADDRESS		  0x2006 /* write      */
#define Z_NES_MMIO_PPU_DATA		  0x2007 /* read/write */
#define Z_NES_MMIO_APU_DMC_CONTROL	  0x4010 /* write      */
#define Z_NES_MMIO_APU_DMC_DIRECT_LOAD	  0x4011 /* write      */
#define Z_NES_MMIO_APU_DMC_SAMPLE_ADDRESS 0x4012 /* write      */
#define Z_NES_MMIO_APU_DMC_SAMPLE_SIZE	  0x4013 /* write      */
#define Z_NES_MMIO_PPU_OAM_DMA		  0x4014 /* write      */
#define Z_NES_MMIO_APU_STATUS		  0x4015 /* read       */
#define Z_NES_MMIO_APU_CONTROL		  0x4015 /* write      */
#define Z_NES_MMIO_CONTROLLER_1		  0x4016 /* read       */
#define Z_NES_MMIO_			  0x4016 /* write      */
#define Z_NES_MMIO_CONTROLLER_2		  0x4017 /* read       */
#define Z_NES_MMIO_APU_FRAME_COUNTER	  0x4017 /* write      */

/* Controller Register */

/*----------------.
| 7 6 5 4 3 2 1 0 |
'-|-|-|-|-|-|-|-|-'
  | | | | | | B A
  | | | | | select
  | | | | start
  | | | up
  | | down
  | left
  right */

Z_DEFINE_PACKED_STRUCTURE ({Z_BIT_FIELD_MEMBERS(8, 8) (
	zuint8 right  :1,
	zuint8 left   :1,
	zuint8 down   :1,
	zuint8 up     :1,
	zuint8 start  :1,
	zuint8 select :1,
	zuint8 b      :1,
	zuint8 a      :1
)}, Z_NESController);

#define Z_NES_CONTROLLER_A      0x01
#define Z_NES_CONTROLLER_B      0x02
#define Z_NES_CONTROLLER_SELECT 0x04
#define Z_NES_CONTROLLER_START  0x08
#define Z_NES_CONTROLLER_UP     0x10
#define Z_NES_CONTROLLER_DOWN   0x20
#define Z_NES_CONTROLLER_LEFT   0x40
#define Z_NES_CONTROLLER_RIGHT  0x80

/* MARK: - Screen */

#define Z_NES_SCREEN_WIDTH    256
#define Z_NES_SCREEN_HEIGHT   240
#define Z_NES_NTSC_SCREEN_FPS 60
#define Z_NES_PAL_SCREEN_FPS  50
#define Z_DENDY_SCREEN_FPS

/* MARK: - Timings */

#define Z_NES_NTSC_MASTER_CLOCK_HZ	21477272 /* 236.25/11 MHz by definition */
#define Z_NES_NTSC_CPU_DIVISOR		12
#define Z_NES_NTSC_CPU_HZ		1789773
#define Z_NES_NTSC_PPU_DIVISOR		4
#define Z_NES_NTSC_PPU_HZ		5369318

#define Z_NES_PAL_MASTER_CLOCK_HZ	26601712 /* 26.6017125 MHz by definition */
#define Z_NES_PAL_CPU_DIVISOR		16
#define Z_NES_PAL_CPU_HZ		1662607
#define Z_NES_PAL_PPU_DIVISOR		5
#define Z_NES_PAL_PPU_HZ		5320343

#define Z_DENDY_MASTER_CLOCK_HZ		26601712 /* 26.6017125 MHz by definition */
#define Z_DENDY_CPU_DIVISOR		15
#define Z_DENDY_CPU_HZ			1773447
#define Z_DENDY_PPU_DIVISOR		Z_NES_PAL_PPU_DIVISOR
#define Z_DENDY_PPU_HZ			Z_NES_PAL_PPU_HZ

/* MARK: - Input */



#endif /* Z_hardware_machine_console_NES_H */
